<?php

namespace Drupal\xnttbrapi\Plugin\ExternalEntities\StorageClient;

use Drupal\Core\Form\FormStateInterface;
use Drupal\external_entities\Plugin\ExternalEntities\StorageClient\Rest;

/**
 * External entities storage client based on a REST API.
 *
 * @ExternalEntityStorageClient(
 *   id = "brapi",
 *   label = @Translation("BrAPI"),
 *   description = @Translation("Retrieves external entities from a Breeding API.")
 * )
 */
class Brapi extends Rest {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(
    array $form,
    FormStateInterface $form_state
  ) {
    $form = parent::buildConfigurationForm($form, $form_state);
    
    // Auto-set some REST parameters.
    $form['response_format'] = [
      '#type' => 'hidden',
      '#value' => 'json',
    ];

    $form['pager']['page_parameter'] = [
      '#type' => 'hidden',
      '#value' => 'page',
    ];

    $form['pager']['page_parameter_type'] = [
      '#type' => 'hidden',
      '#value' => 'pagenum',
    ];

    $form['pager']['page_size_parameter'] = [
      '#type' => 'hidden',
      '#value' => 'pageSize',
    ];

    $form['pager']['page_size_parameter_type'] = [
      '#type' => 'hidden',
      '#value' => 'pagesize',
    ];
    
    return $form;
  }

  /**
   * Returns the BrAPI call URL.
   *
   * @param string $id
   *   A BrAPI entity identifier or an empty string if none for listing.
   *
   * @return string
   *   The BrAPI endpoint URL qualified with the given identifier if one.
   */
  public function getEndpoint(string $id = '') :string {
    $brapi_url = $this->configuration['endpoint'];
    // Add the entity identifier to the URL.
    if (preg_match('#\{\w+\}#', $brapi_url)) {
      // Use specified placeholder if one.
      // @todo: Maybe manage multiple placeholders and identifiers, for instance
      // "maps/{mapDbId}/positions/{linkageGroupName}" with "42/806"?
      if (empty($id)) {
        $brapi_url = preg_replace('#\{\w+\}.*#', '', $brapi_url);
      }
      else {
        $brapi_url = preg_replace('#\{\w+\}#', $id, $brapi_url);
      }
    }
    else {
      // Append otherwise.
      $brapi_url = rtrim($brapi_url, '/') . '/' . $id;
    }

    return $brapi_url;
  }

  /**
   * {@inheritdoc}
   */
  public function load($id) {
    
    $brapi_url = $this->getEndpoint($id);
    try {
      $response = $this->httpClient->request(
        'GET',
        $brapi_url,
        [
          'headers' => $this->getHttpHeaders(),
          'query' => $this->getSingleQueryParameters($id),
        ]
      );
      
      $format = $this->configuration['response_format'];
      // Remove BOM.
      $body = ltrim($response->getBody(), "\xEF\xBB\xBF");

      $result = $this
        ->getResponseDecoderFactory()
        ->getDecoder($format)
        ->decode($body)
      ;
    }
    catch (\GuzzleHttp\Exception\ServerException $e) {
      \Drupal::messenger()->addWarning(
        $this->t('Failed to query BrAPI endpoint. See logs for details.')
      );
      $response = $e->getResponse();
      $response_body = $response->getBody()->getContents();
      \Drupal::logger('xnttbrapi')->warning(
        'Failed to query BrAPI endpoint: '
        . $response_body
      );
    }

    if (!empty($result['result'])) {
      return $result['result'];
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function query(array $parameters = [], array $sorts = [], $start = NULL, $length = NULL) {
    try {
      $response = $this->httpClient->request(
        'GET',
        $this->getEndpoint(),
        [
          'headers' => $this->getHttpHeaders(),
          'query' => $this->getListQueryParameters($parameters, $start, $length),
        ]
      );

      $format = $this->configuration['response_format'];
      $body = ltrim($response->getBody(), "\xEF\xBB\xBF");

      $results = $this
        ->getResponseDecoderFactory()
        ->getDecoder($format)
        ->decode($body);
    }
    catch (\GuzzleHttp\Exception\ServerException $e) {
      \Drupal::messenger()->addWarning(
        $this->t('Failed to query BrAPI endpoint. See logs for details.')
      );
      $response = $e->getResponse();
      $response_body = $response->getBody()->getContents();
      \Drupal::logger('xnttbrapi')->warning(
        'Failed to query BrAPI endpoint: '
        . $response_body
      );
    }

    $items = [];
    if (!empty($results['result']['data'])) {
      $items = array_values($results['result']['data']);
    }

    return $items;
  }

  /**
   * {@inheritdoc}
   */
  public function countQuery(array $parameters = []) {
    $response = $this->httpClient->request(
      'GET',
      $this->getEndpoint(),
      [
        'headers' => $this->getHttpHeaders(),
        'query' => $this->getListQueryParameters($parameters, 0, 1),
      ]
    );

    $format = $this->configuration['response_format'];
    $body = ltrim($response->getBody(), "\xEF\xBB\xBF");

    $results = $this
      ->getResponseDecoderFactory()
      ->getDecoder($format)
      ->decode($body);

    $count = 0;
    // Tries to get data count from the API.
    if (!empty($results['metadata']['pagination']['totalCount'])) {
      $count = $results['metadata']['pagination']['totalCount'];
    }
    else {
      // Fallback to default if it failed.
      $count = parent::countQuery($parameters);
    }

    return $count;
  }

}
